package it.pf.base;


import it.pf.base.ObjectBean;
import it.pf.dbaccess.DataMapper;

import java.io.Serializable;
import java.util.Hashtable;

import javax.sql.DataSource;

/**
 * Estensione di ObjectBean che mantiene la configurazione
 * dell'applicazione web inizializzandola da un file xml.<br>
 * Si tratta di un singleton e pertanto se ne acquisisce l'istanza
 * dal metodo getInstance.<br>
 *  
 * @author Fabbri_P
 *
 */
public final class ConfigSingleton extends ObjectBean implements Serializable {
	private static ObjectBean _resources = null;
	private static DataSource _ds = null;
	private static Hashtable _datasources = null;
	private static DataMapper _mapper = null;
    private static String _rootPath = null;
    private static String _ConfigFile = null;
    private static ConfigSingleton _instance = null;
    private static final String ROOT_PATH = "";
    private static final String CONFIG_FILE = "/WEB-INF/conf/config.xml";

   /**
    * costruttore: inizializza l'OB dal file xml ConfigFile
    * @param ConfigFile
    * @throws Exception
    */
	private ConfigSingleton(String ConfigFile) throws Exception {  
    	super(ObjectBean.fromXMLFile(ConfigFile));   
    }
    
	/**
	 * costruttore di default
	 * @throws Exception
	 */
    private ConfigSingleton() throws Exception {  
    	super(_resources);
    }
    
	/**
	 * consente di acquisire l'istanza del singleton
	 * @return ConfigSingleton
	 */
    public static ConfigSingleton getInstance() {
        if (_instance == null) {
            synchronized(ConfigSingleton.class) {
                if (_instance == null) {
                    try {
                    	if (_resources == null)
                    		_instance = new ConfigSingleton(getConfigFileName());
                    	else 
                    		_instance = new ConfigSingleton();
                    } // try
                    catch (Exception ex) {
                        System.out.println("ConfigSingleton::getInstance: errore creazione configuratore [" +
                            getConfigFileName() + "]");
                        ex.printStackTrace();
                    } 
                } 
            } 
        } 
        return _instance;
    } 

    public static boolean isInitialized() {
        if (_instance == null) 
        	return false;
        else 
        	return true;
    }
    
	/**
	 * fa in modo che il Singleton sia serializzabile come singola 
	 * istanza (sempre la stessa)
	 * With the readResolve() method in place, deserialization 
	 * results in the one (and only one) object -- the same object 
	 * as produced by calls to the getInstance() method. 
	 * If you don't provide a readResolve() method, an instance 
	 * of the object is created each time you deserialize the object
	 * @return Object
	 */
    private Object readResolve() {
        return _instance;
    } 
    
	/**
	 * rilascia la configurazione settando a null le risorse
	 * e l'istanza
	 */
    public static void release() {
        if (_instance != null) {
            synchronized(ConfigSingleton.class) {
                _instance = null;
                _resources = null;	
            }
        } 
    }

	/**
	 * acquisisce il RootPath dell'applicazione 
	 * @return String
	 */
    public static String getRootPath() {
        if (_rootPath == null) {
            synchronized(ConfigSingleton.class) {
                if (_rootPath == null)
                    _rootPath = System.getProperty("ROOT_PATH");
                if (_rootPath == null)
                    _rootPath = ROOT_PATH;
            } 
        } 
        return _rootPath;
    } 

	/**
	 * imposta il RootPath dell'applicazione
	 * @param rootPath
	 */
    public static void setRootPath(String rootPath) {
        synchronized(ConfigSingleton.class) {
            _rootPath = rootPath;
        }
    } 
    
	/**
	 * ritorna il nome del file xml di configurazione
	 * @return String
	 */
    public static String getConfigFileName() {
        if (_ConfigFile == null) {
            synchronized(ConfigSingleton.class) {
                if (_ConfigFile == null)
                    _ConfigFile = System.getProperty("CONFIG_FILE");
                if (_ConfigFile == null)
                    _ConfigFile = CONFIG_FILE;
            } 
        } 
        return _ConfigFile;
    } 

	/**
	 * imposta il nome del file xml di configurazione
	 * @param ConfigFile
	 */
    public static void setConfigFileName(String ConfigFile) {
        synchronized(ConfigSingleton.class) {
            _ConfigFile = ConfigFile;
        }
    } 
    
	/**
	 * imposta l'OB  delle risorse inizializzate
	 * @param resources
	 */
    public static void setResources(ObjectBean resources) {
        synchronized(ConfigSingleton.class) {
            _resources = resources;
            try {
	            if (_mapper == null) {
	        		ObjectBean t = ConfigSingleton.getResources().getFilteredAttribute("CONFIG:DATA_BASE:PARAMETER","NAME","dbname");
	        		if (t != null) {
		        		String dbname = (String)t.getAttributeValue("value");
		        		if (dbname.length() > 2) {
			        		String suffix = dbname.substring(0,3);
							_mapper = (DataMapper)Class.forName("it.pf.dbaccess.DataMapper4"+suffix).newInstance();
		        		} else 
		        			_mapper = null;
	        		} else
	        			_mapper = null;
	            }
			} catch (InstantiationException e) {
				e.printStackTrace();
				_mapper = null;
			} catch (IllegalAccessException e) {
				e.printStackTrace();
				_mapper = null;
			} catch (ClassNotFoundException e) {
				e.printStackTrace();
				_mapper = null;
			}
        } 
    }     
	  
	/**
	 * acquisisce l'OB delle risorse inizializzate
	 * @return ObjectBean
	 */
    public static ObjectBean getResources() {
        synchronized(ConfigSingleton.class) {
            return _resources;
        } 
    }

	/**
	 * ritorna il DataSource di default
	 * @return DataSource
	 */
	public static DataSource getDataSource() {
		return _ds;
	}
	
	/**
	 * imposta il DataSource inizializzato (datasource di default)
	 * @param _ds
	 */
	public static void setDataSource(DataSource _ds) {
		ConfigSingleton._ds = _ds;
	}

	public static DataMapper get_mapper() {
		return _mapper;
	}
	
	/**
	 * aggiunge eventuali altri data sources
	 * @param name
	 * @param _ds
	 */
	public static void addDataSource(String name, DataSource _ds) {
		if (_datasources == null)
			_datasources = new Hashtable();
		_datasources.put(name, _ds);
	}
	
	/**
	 * ritorna lo specifico datasource
	 * @param name
	 * @return
	 */
	public static DataSource getDataSource(String name) {
		if (_datasources == null)
			return null;
		else
			return (DataSource)_datasources.get(name);
	}
	     
} 
