package it.pf.base;

import it.pf.dbaccess.DataField;
import it.pf.dbaccess.DataRow;
import it.pf.dbaccess.DataSheet;

import java.beans.BeanInfo;
import java.beans.IntrospectionException;
import java.beans.Introspector;
import java.beans.MethodDescriptor;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.Serializable;
import java.io.StringReader;
import java.io.StringWriter;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;
import java.util.Vector;

import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.apache.xerces.dom.DocumentImpl;
import org.apache.xerces.parsers.SAXParser;
import org.w3c.dom.CDATASection;
import org.w3c.dom.CharacterData;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Text;
import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;

/**
 * contenitore formato da:<br>
 * _name	nome dell'oggetto<br>
 * _content elemento descrittivo<br>
 * _table	vettore di oggetti<br>
 * <br>
 * ogni oggetto del vettore &egrave; un ObjectBeanAttribute<br>
 * costituito da:<br>
 * 	chiave	una stringa<br> 
 * 	valore	un oggetto che pu&ograve; essere di 3 tipi:<br>
 * 							ObjectBean<br>
 * 							String<br>
 * 							DataRow<br>
 * <br>
 * In tal modo ObjectBean (OB) � in grado di rimappare una struttura xml
 * con qualunque contenuto. L'unica eccezione &egrave; il DataRow in corrispondenza
 * del valore di un ObjectBeanAttribute (OBA), ogni DataRow si compone di DataFields
 * <br>
 * Si pu� costruire un ObjectBean in memoria o a partire da un file xml
 * &egrave; possibile riottenere il file xml a partire dall'ObjectBean
 * per generare l'OB a partire da un file xml viene utilizzato un normale
 * SAX parser il cui handler &egrave; ObjectBeanHandler
 * <br>
 * esempi di utilizzo:
 * <br>
 * inizializzazione da file xml:<br>
 *  ObjectBean.fromXMLFile(ConfigFile)   -- metodo statico
 * <br>
 * ricerca di un elemento annidato nella struttura<br>
 * 	ObjectBean menu = ConfigSingleton.getResources().getFilteredAttribute("CONFIG:MENUS:MENU","NAME","Main");
 * <br>
 * altro esempio di ricerca:<br>
 *	ObjectBean o = ConfigSingleton.getResources().getFilteredAttribute("CONFIG:VARIE:PARAMETER","name","root_dir");<br>
 *	if (o != null) {<br>
 *		root = o.getAttributeValueAsString("value");<br>
 *		if (root == null)<br>
 *			root = System.getProperty("java.io.tmpdir");<br>
 *	} else<br>
 *		root = System.getProperty("java.io.tmpdir");<br>
 *
 * @author Fabbri_P
 *
 */
public class ObjectBean implements Serializable,Cloneable {

	private Document _doc;
	private Element _elem;
	
	private String _name;
	private Vector _table;
	private String _content;
	
	/**
	 * costruisce l'OB con un nome e un vettore vuoti
	 */
	public ObjectBean(String name) {
		_table = new Vector();
		_name = name;
	}

	/**
	 * costruisce l'OB a partire da un'altro OB 
	 */
	public ObjectBean(ObjectBean bean) {
		_table = (Vector)bean.get_table();
		_content = bean.get_content();
		_name = bean.get_name();
	}
	
	/**
	 * aggiunge il contenuto della table di un'altro OB
	 * al corrente
	 * @param o
	 */
	public void add(ObjectBean o) {
		Vector ov = (Vector)o.get_table();
		for (int i=0; i<ov.size(); i++) {
			_table.add(ov.elementAt(i));
		}
	}
	
	/**
	 * ritorna l'ObjectBeanAttribute (OBA) che ha per chiave
	 * la stringa passata come parametro al metodo  
	 * @param  key
	 * @return ObjectBeanAttribute
	 */
	public ObjectBeanAttribute getAttribute(String key) {
		for (int i=0; i<_table.size(); i++) {
			ObjectBeanAttribute v = (ObjectBeanAttribute)_table.elementAt(i);
			if ((v.get_key()).equalsIgnoreCase(key))
				return v;
		}
		return null;
	}
	
	/**
	 * ritorna il valore dell'ObjectBeanAttribute (OBA) che ha per chiave
	 * la stringa passata come parametro al metodo
	 * @param  key
	 * @return Object  
	 */
	public Object getAttributeValue(String key) {
		for (int i=0; i<_table.size(); i++) {
			ObjectBeanAttribute v = (ObjectBeanAttribute)_table.elementAt(i);
			if ((v.get_key()).equalsIgnoreCase(key))
				return v.get_value();
		}
		return null;
	}

	
	/**
	 * ritorna il valore dell'ObjectBeanAttribute (OBA) che ha per chiave
	 * la stringa passata come parametro al metodo e lo ritorna come stringa
	 * se questo &egrave; possibile altrimenti ritorna null
	 * @param  key
	 * @return String  
	 */
	public String getAttributeValueAsString(String key) {
		for (int i=0; i<_table.size(); i++) {
			ObjectBeanAttribute v = (ObjectBeanAttribute)_table.elementAt(i);
			if ((v.get_key()).equalsIgnoreCase(key)) {
				Object r = v.get_value();
				if (r instanceof java.lang.String) 
					return (String)r;
				else
					return null;
			}
		}
		return null;
	}

	/**
	 * ritorna gli ObjectBeanAttribute (OBA) dell'OB come List
	 * @return List
	 */
	public List getAttributesAsList() {
		List values = new ArrayList();
		for (int i=0; i<_table.size(); i++) {
			ObjectBeanAttribute v = (ObjectBeanAttribute)_table.elementAt(i);
			values.add(v);
		}
		return values;
	}

	/**
	 * ritorna gli ObjectBeanAttribute (OBA) dell'OB con data chiave come List
	 * @param attribute
	 * @return List
	 */
	public List getAttributesAsList(String attribute) {
		List values = new ArrayList();
		for (int i=0; i<_table.size(); i++) {
			ObjectBeanAttribute v = (ObjectBeanAttribute)_table.elementAt(i);
			if (v.get_key().equalsIgnoreCase(attribute))
				values.add(v);
		}
		return values;
	}
	
	/**
	 * Imposta un attributo dell'OB a partire da chiave e valore (come Stringa)
	 * generando un OBA aggiunto al vettore
	 * @param key
	 * @param value
	 */
    public void setAttribute(String key, String value) {
		_table.add(new ObjectBeanAttribute(key,value));
	}
	
    /**
	 * Imposta un attributo dell'OB a partire da chiave e valore (come Object)
	 * generando un OBA aggiunto al vettore
     * @param key
     * @param value
     */
	public void setAttribute(String key, Object value) {
		_table.add(new ObjectBeanAttribute(key,value));
	}
	
	/**
	 * Imposta un OB come OBA di questo OB
	 * generando un OBA con nome = nome OB parametro
	 * @param bean
	 */
	public void setAttribute(ObjectBean bean) {
		_table.add(new ObjectBeanAttribute(bean.get_name(),bean));
	}
	
	/**
	 * Imposta il contenuto descrittivo dell'OB
	 * @param content
	 */
	public void setCharacters(String content) {
		_content = content.trim();
	}

	/**
	 * ritorna il contenuto descrittivo dell'OB
	 * @return String
	 */
	public String get_content() {
		return _content;
	}
	
	/**
	 * Imposta il contenuto descrittivo dell'OB
	 * @param _content
	 */
	public void set_content(String _content) {
		this._content = _content;
	}
	
	/**
	 * ritorna il nome dell'OB
	 * @return String
	 */
	public String get_name() {
		return _name;
	}
	
	/**
	 * Imposta il nome dell'OB
	 * @param _name
	 */
	public void set_name(String _name) {
		this._name = _name;
	}
	
	/**
	 * ritorna come List il vettore di elementi associato all'OB
	 * @return List
	 */
	public List get_table() {
		return (List)_table;
	}
	
	/**
	 * ritorna il numero di oggetti nel vettore di elementi associato all'OB
	 * @return int
	 */
	public int get_size() {
		return _table.size();
	}

	/**
     * Ritorna OB ottenuto dal parsing
     * dell'<code>InputSource</code> <em>stream</em>.
     */
    public static ObjectBean fromXMLStream(InputSource stream) throws Exception {
        if (stream == null) {
            return null;
        }
        ObjectBean ObjectBean = null;
        XMLReader parser = new SAXParser();
        try {
            ObjectBeanContentHandler contentHandler =
                new ObjectBeanContentHandler();
            parser.setContentHandler(contentHandler);
            parser.parse(stream);
            ObjectBean = contentHandler.getObjectBean();
        } catch (Exception ex) {
			ex.printStackTrace();
			System.out.println("Inizializzazione interrotta.");
            throw new Exception("Impossibile elaborare lo stream XML:");
        } 
        return ObjectBean;
    } 

    /**
     * Ritorna OB ottenuto dal parsing
     * dell'<code>InputString</code> 
     */
    public static ObjectBean fromXMLString(String stream) throws Exception {
        if (stream == null) {
            return null;
        }
        return fromXMLStream(new InputSource(new StringReader(stream)));
    } 
 
    /**
     * Ritorna OB ottenuto dal parsing
     * del file <em>xmlObjectBean</em>.
     */
    public static ObjectBean fromXMLFile(String xmlObjectBean)
        throws Exception {
        if (xmlObjectBean == null) {
            throw new Exception("XMLObjectBean non valido");
        }
        ObjectBean ObjectBean = null;
        try {
            InputSource stream = new InputSource(new FileReader(xmlObjectBean));
            ObjectBean = fromXMLStream(stream);
        } catch (FileNotFoundException ex) {
            throw new Exception("File non trovato");
        }
        return ObjectBean;
    } 


    /**
     * Ritorna OB ottenuto dal DataSheet
     */
    public static ObjectBean fromDataSheet(DataSheet ds, String obname)
        throws Exception {
        if (ds == null) {
            throw new Exception("DataSheet non valido");
        }
        ObjectBean ObjectBean = new ObjectBean(obname);
        for (int i=0; i<ds.getRowSize(); i++) {
            ObjectBean.setAttribute("ROW",ds.getRow(i));
        }
        return ObjectBean;
    } 

    /**
     * Ritorna OB ottenuto dal bean
     */
    public static ObjectBean fromBean(Object bean)  {
    	ObjectBean ob = null;
    	if (bean == null)
    		return null;
    	String beanName = bean.getClass().getName();
    	if (beanName.lastIndexOf(".") >= 0)
    		beanName = beanName.substring(beanName.lastIndexOf(".")+1);
    	//beanName = beanName.toUpperCase();
    	ob = new ObjectBean(beanName);
    	BeanInfo bi;
		try {
			bi = Introspector.getBeanInfo(bean.getClass());
		} catch (IntrospectionException e) {
			e.printStackTrace();
			return null;
		}
		String ns = null;   //namespace
    	MethodDescriptor[] mda = bi.getMethodDescriptors();
    	for (int i=0; i<mda.length; i++) {
    		MethodDescriptor md = mda[i];
    		if (md.getName().equals("Namespace")) {
    			Method m = md.getMethod();
    			int mod = m.getModifiers();
    			if (mod == Modifier.PUBLIC) {
	    			Object o = null;
					try {
						o = m.invoke(bean,null);
					} catch (IllegalArgumentException e) {
						e.printStackTrace();
						return null;
					} catch (IllegalAccessException e) {
						e.printStackTrace();
						return null;
					} catch (InvocationTargetException e) {
						e.printStackTrace();
						return null;
					}
					ns = (String)o;  //set namespace value
					ob.set_name(ns+":"+beanName);
    			}
    		}
    	}
    	for (int i=0; i<mda.length; i++) {
    		MethodDescriptor md = mda[i];
    		if (md.getName().startsWith("get")) {
    			Method m = md.getMethod();
    			int mod = m.getModifiers();
    			if (mod == Modifier.PUBLIC) {
	    			Object o = null;
					try {
						o = m.invoke(bean,null);
					} catch (IllegalArgumentException e) {
						e.printStackTrace();
						return null;
					} catch (IllegalAccessException e) {
						e.printStackTrace();
						return null;
					} catch (InvocationTargetException e) {
						e.printStackTrace();
						return null;
					}
					if (o == null) {
						String attName = m.getName().substring(3);
						if (ns != null)
							attName = ns+":"+attName;
						ObjectBean oba = new ObjectBean(attName);
						oba.set_content("null");
						ob.setAttribute(oba);
					} else if (o instanceof String) {
						String attName = m.getName().substring(3);
						if (ns != null)
							attName = ns+":"+attName;
						ObjectBean oba = new ObjectBean(attName);
						oba.set_content((String)o);
						ob.setAttribute(oba);
					} else if (isWrapper(o)) {
						String attName = m.getName().substring(3);
						if (ns != null)
							attName = ns+":"+attName;
						ObjectBean oba = new ObjectBean(attName);
						oba.set_content(getString(o));
						ob.setAttribute(oba);
					} else {
						String attName = m.getName().substring(3);
						if (ns != null)
							attName = ns+":"+attName;
						ObjectBean oba = new ObjectBean(attName);
						oba = fromBean(o);
						ob.setAttribute(oba);
					}
    			}
    		}
    	}
    	return ob;
    } 
	
    /**
     * testa se l'oggetto passato � wrapper di uno dei tipi fondamentali:
     * Integer per int, Float per float ecc.
     * @param o
     * @return
     */
    private static boolean isWrapper(Object o) {
    	if (o instanceof Integer)
    		return true;
    	else if (o instanceof Double)
    		return true;
    	else if (o instanceof Float)
    		return true;
    	else if (o instanceof Long)
    		return true;
    	else if (o instanceof BigDecimal)
    		return true;
    	else if (o instanceof BigInteger)
    		return true;
    	else if (o instanceof Byte) 
    		return true;
    	else if (o instanceof Character)
    		return true;
    	else 
    		return false;
    }

    /**
     * ritorna la stringa corrispondente all'oggetto passato 
     * se questo � wrapper di uno dei tipi fondamentali:
     * Integer per int, Float per float ecc.
     * @param o
     * @return
     */
    private static String getString(Object o) {
    	if (o instanceof Integer)
    		return o.toString();
    	else if (o instanceof Double)
    		return o.toString();
    	else if (o instanceof Float)
    		return o.toString();
    	else if (o instanceof Long)
    		return o.toString();
    	else if (o instanceof BigDecimal)
    		return o.toString();
    	else if (o instanceof BigInteger)
    		return o.toString();
    	else if (o instanceof Byte) 
    		return o.toString();
    	else if (o instanceof Character)
    		return o.toString();
    	else 
    		return "null";
    }
    
    /**
	 * ritorna l'xml corrispondente all'OB
	 * @return String
	 */
	public String getXML() {
		return getXML(false);
	}
	
	/**
	 * ritorna l'xml corrispondente all'OB
	 * impostato come vera struttura XMLs
	 * @return String
	 */
	public String getXML(boolean trueXML) {
		toDOM(trueXML);
		return outXML(_doc);
	}

	/**
	 * ritorna il text corrispondente all'OB: 
	 * sequenza di caratteri in formato stringa
	 * @return String
	 */
	public String toString() {
		toDOM(true);
		return outTEXT(_doc);
	}

	/**
	 * ritorna il DOM corrispondente all'OB
	 * @return String
	 */
	public Document getDOM() {
		toDOM(true);
		return _doc;
	}

	
	/**
	 * costruisce una struttura DOM in memoria a partire dall'OB
	 * il DataRow viene generato come unico nodo con attributi = DataFields
	 * set trueXML = true allora il DataRow viene generato 
	 * con n nodi ciascuno corrispondente ad un DataField
	 */
	private void toDOM(boolean trueXML) {
		_doc = new DocumentImpl();
		Element root = _doc.createElement(this.get_name());
		if (this.get_content() != null)
			if (this.get_content().length() > 0)
				root.setAttribute("text",this.get_content());
		Vector v = (Vector)this.get_table();
		for (int i=0; i<v.size(); i++) {
			ObjectBeanAttribute ba = (ObjectBeanAttribute)v.elementAt(i);
			Object value = ba.get_value();
			if (value instanceof String)
				root.setAttribute(ba.get_key(),(String)value);
			else if (value instanceof ObjectBean)
				processChild(root,_doc,(ObjectBean)value);
			else if (value instanceof DataRow) {
				if (trueXML)
					processChildXML(root,_doc,(DataRow)value);
				else
					processChild(root,_doc,(DataRow)value);	
			} else
				root.setAttribute(ba.get_key(),value.toString());
		}
		_doc.appendChild(root);		
	}

	
	/**
	 * processa un elemento dell'OB per generare un nodo del DOM
	 * @param parent
	 * @param doc
	 * @param bean
	 */
	private void processChild(Element parent, Document doc, ObjectBean bean) {
		Element nodo = doc.createElement(bean.get_name());
		String content = bean.get_content();
		if (content != null)
			if (content.length() > 0)
				nodo.appendChild(doc.createTextNode(content));
		Vector v = (Vector)bean.get_table();
		for (int i=0; i<v.size(); i++) {
			ObjectBeanAttribute ba = (ObjectBeanAttribute)v.elementAt(i);
			Object value = ba.get_value();
			if (value instanceof String)
				nodo.setAttribute(ba.get_key(),(String)value);
			else if (value instanceof ObjectBean)
				processChild(nodo,doc,(ObjectBean)value);	
			else if (value instanceof DataRow)
				processChild(nodo,doc,(DataRow)value);	
			else
				nodo.setAttribute(ba.get_key(),value.toString());
		}
		parent.appendChild(nodo);
	}

	/**
	 * processa un elemento dell'OB per generare un nodo del DOM
	 * nel caso che l'oggetto rilevato sia del tipo <code>DataRow</code>
	 * @param parent
	 * @param doc
	 * @param bean
	 */
	private void processChild(Element parent, Document doc, DataRow bean) {
		Element nodo = doc.createElement(Constants.ROW);
		for (int i=0; i<bean.getColumnSize(); i++) {
			DataField df = bean.getColumn(i);
			String val = df.getStringValue();
			if (df.getObjectValue() == null)
				val = "null";
			else {
				if (val != null)
					val = val.trim();
			}
			nodo.setAttribute(df.getName(),val);
		}
		parent.appendChild(nodo);
	}

	/**
	 * processa un elemento dell'OB per generare un nodo del DOM
	 * nel caso che l'oggetto rilevato sia del tipo <code>DataRow</code>
	 * in questo metodo ogni attributo diventa un nuovo nodo.
	 * @param parent
	 * @param doc
	 * @param bean
	 */
	private void processChildXML(Element parent, Document doc, DataRow bean) {
		Element nodo = doc.createElement(Constants.ROW);
		for (int i=0; i<bean.getColumnSize(); i++) {
			DataField df = bean.getColumn(i);
			Element attribute = doc.createElement(df.getName());
			String val = df.getStringValue();
			if (df.getObjectValue() == null)
				val = "";
			else {
				if (val != null)
					val = val.trim();
			}
			attribute.appendChild(doc.createTextNode(val));
			nodo.appendChild(attribute);
		}
		parent.appendChild(nodo);
	}

	/**
     * trasforma doc in xml e lo scrive in output
     */
    private String outXML(Document doc) {
    	try {
	 		StringWriter stringOut = new StringWriter();
		    TransformerFactory tFact = TransformerFactory.newInstance();
	    	Transformer tForm = tFact.newTransformer();
		    tForm.setOutputProperty(OutputKeys.METHOD,"xml");
			tForm.setOutputProperty(OutputKeys.INDENT,"yes");
	    	tForm.transform(new DOMSource(doc),new StreamResult(stringOut));
	    	String stringXML = stringOut.toString();
			return stringXML;
    	} catch (Exception e) {
			System.out.println("exception in transform:"+e.getMessage());
			return null;
    	}
    }
	
	/**
     * trasforma doc in text e lo scrive in output
     */
    private String outTEXT(Document doc) {
    	try {
	 		StringWriter stringOut = new StringWriter();
		    TransformerFactory tFact = TransformerFactory.newInstance();
	    	Transformer tForm = tFact.newTransformer();
		    tForm.setOutputProperty(OutputKeys.METHOD,"text");
			tForm.setOutputProperty(OutputKeys.INDENT,"yes");
	    	tForm.transform(new DOMSource(doc),new StreamResult(stringOut));
	    	String stringTXT = stringOut.toString();
			return stringTXT;
    	} catch (Exception e) {
			System.out.println("exception in transform:"+e.getMessage());
			return null;
    	}
    }

    
    /**
	 * costruisce un file di Properties con i contenuti del bean
	 */
	public Properties getAttributesAsProperties() {
		Properties p = new Properties();
		String key = this.get_name();
		if (this.get_content() != null)
			if (this.get_content().trim().length() > 0)
				p.setProperty(key,this.get_content());
		if (this.get_table() != null) {
			Vector v = (Vector)this.get_table();
			for (int i=0; i<v.size(); i++) {
				ObjectBeanAttribute ba = (ObjectBeanAttribute)v.elementAt(i);
				Object value = ba.get_value();
				if (value instanceof String) {
					String k = key+"."+ba.get_key();
					p.setProperty(k,(String)value);
				} else if (value instanceof ObjectBean)
					processChildOB(key,p,(ObjectBean)value);
				else if (value instanceof DataRow)
					processChildOB(key,p,(DataRow)value);	
			}
		}
		return p;
	}

	/**
	 * processa un elemento dell'OB per aggiungere i valori alle properties
	 * l'oggetto ricevuto � un ObjectBean
	 * @param parent
	 * @param doc
	 * @param bean
	 */
	private void processChildOB(String key, Properties p, ObjectBean bean) {
		key += "."+bean.get_name();
		if (bean.get_content() != null)
			if (bean.get_content().trim().length() > 0)
				p.setProperty(key,bean.get_content());
		if (bean.get_table() != null) {
			Vector v = (Vector)bean.get_table();
			for (int i=0; i<v.size(); i++) {
				ObjectBeanAttribute ba = (ObjectBeanAttribute)v.elementAt(i);
				Object value = ba.get_value();
				if (value instanceof String) {
					String k = key+"."+ba.get_key();
					p.setProperty(k,(String)value);
				} else if (value instanceof ObjectBean)
					processChildOB(key,p,(ObjectBean)value);
				else if (value instanceof DataRow)
					processChildOB(key,p,(DataRow)value);	
			}
		}
	}
	
	/**
	 * processa un elemento dell'OB per aggiungere i valori alle properties
	 * l'oggetto ricevuto � un DataRow
	 * @param parent
	 * @param doc
	 * @param bean
	 */
	private void processChildOB(String key, Properties p, DataRow bean) {
		key += "."+Constants.ROW;
		for (int i=0; i<bean.getColumnSize(); i++) {
			DataField df = bean.getColumn(i);
			String val = df.getStringValue();
			if (df.getObjectValue() == null)
				val = "null";
			else {
				if (val != null)
					val = val.trim();
			}
			p.setProperty(df.getName(),val);
		}
	}
	
	
	/**
	 * ricerca il valore di un attributo in profondit&agrave; nella struttura e
	 * ritorna l'OB che lo contiene
	 * @param key
	 * @param attribute
	 * @param value
	 * @return OB o null se non trovato attributo nella struttura
	 */
	public ObjectBean getFilteredAttribute(String key, String attribute, String value) {
		ObjectBean searchBean = null;
        boolean deepSearch = (key.indexOf(':') != -1);
		if (!deepSearch) {
			if (hasAttributeValue(this,attribute,value))
				return this;
			else
				return null;
		} else {
			String[] names = key.split(":");
			if ( !names[0].equalsIgnoreCase(this.get_name()) ) {
				//System.out.println("No on this bean.");
				return null;
			} else {
				searchBean = new ObjectBean(this);
				for (int j=1; j < names.length; j++) {
					Vector v = (Vector)searchBean.get_table();
					for (int i=0; i<v.size(); i++) {
						ObjectBeanAttribute ba = (ObjectBeanAttribute)v.elementAt(i);
						if ( (ba.get_key().equalsIgnoreCase(names[j])) ) {
							searchBean = new ObjectBean((ObjectBean)ba.get_value());
							if (hasAttributeValue(searchBean,attribute,value))
								return searchBean;
						}
					}					
				}
				return null;
			}
		}
	}

	
	/**
	 * ritorna l'OB corrispondente alla chiave di ricerca
	 * @param key
	 * @return OB o null se non trovata chiave 
	 */
	public ObjectBean getFilteredAttribute(String key) {
		ObjectBean searchBean = null;
        boolean deepSearch = (key.indexOf(':') != -1);
		if (!deepSearch) {
			return this;
		} else {
			String[] names = key.split(":");
			if ( !names[0].equalsIgnoreCase(this.get_name()) ) {
				//System.out.println("No on this bean.");
				return null;
			} else {
				searchBean = new ObjectBean(this);
				for (int j=1; j < names.length; j++) {
					Vector v = (Vector)searchBean.get_table();
					for (int i=0; i<v.size(); i++) {
						ObjectBeanAttribute ba = (ObjectBeanAttribute)v.elementAt(i);
						if ( (ba.get_key().equalsIgnoreCase(names[j])) ) {
							if (ba.get_value() instanceof ObjectBean) {
								if (j == (names.length - 1))
									return (ObjectBean)ba.get_value();
								else {
									searchBean = (ObjectBean)ba.get_value();
								}
							} else
								return null;
						}
					}					
				}
				return null;
			}
		}
	}

	/**
	 * ritorna l'OB corrispondente alla chiave di ricerca
	 * @param key
	 * @return OB o null se non trovata chiave 
	 */
	public List getFilteredAttributesAsList(String key) {
		List values = new ArrayList();		
		ObjectBean searchBean = null;
        boolean deepSearch = (key.indexOf(':') != -1);
		if (!deepSearch) {
			values.add(this);
		} else {
			String[] names = key.split(":");
			if ( !names[0].equalsIgnoreCase(this.get_name()) ) {
				//System.out.println("No on this bean.");
				return null;
			} else {
				searchBean = new ObjectBean(this);
				for (int j=1; j < names.length; j++) {
					Vector v = (Vector)searchBean.get_table();
					for (int i=0; i<v.size(); i++) {
						ObjectBeanAttribute ba = (ObjectBeanAttribute)v.elementAt(i);
						if ( (ba.get_key().equalsIgnoreCase(names[j])) ) {
							if (ba.get_value() instanceof ObjectBean) {
								if (j == (names.length - 1))
									values.add(ba.get_value());
								else {
									searchBean = (ObjectBean)ba.get_value();
								}
							} 
						}
					}					
				}
			}
		}
		return values;
	}
	
	/**
	 * valuta se l'OB ha l'attributo richiesto
	 * @param bean
	 * @param attribute
	 * @return true se vero altrimenti false
	 */
	private boolean hasAttribute(ObjectBean bean, String attribute) {
		Vector v = (Vector)bean.get_table();
		for (int i=0; i<v.size(); i++) {
			ObjectBeanAttribute ba = (ObjectBeanAttribute)v.elementAt(i);
			if (ba.get_key().equalsIgnoreCase(attribute)) 
				return true;
		}
		return false;
	}
	
	/**
	 * valuta se l'OB ha l'attributo richiesto con il valore stringa richiesto
	 * @param bean
	 * @param attribute
	 * @param value
	 * @return true se vero altrimenti false
	 */
	private boolean hasAttributeValue(ObjectBean bean, String attribute, String value) {
		Vector v = (Vector)bean.get_table();
		for (int i=0; i<v.size(); i++) {
			ObjectBeanAttribute ba = (ObjectBeanAttribute)v.elementAt(i);
			if ( (ba.get_key().equalsIgnoreCase(attribute)) &&
				 (((String)ba.get_value()).equalsIgnoreCase(value)) )
					return true;
		}
		return false;
	}

	public void set_table(Vector _table) {
		this._table = _table;
	}
	
}
