package it.pf.scheduler;

import java.util.*;
import java.text.*;

/**
 * Questa classe costruisce un TimerTask e lo schedula all'ora desiderata
 * il task viene creato a partire dal nome della classe di esecuzione che
 * deve implementare l'interfaccia <code>Command</code> e dall'array di 
 * parametri necessari alla classe di esecuzione per agire.
 * @author Fabbri_P
 *
 */
public class Scheduler extends Thread {
    private Timer timer = null;

	/**
	 * costruttore: inizializza un oggetto Timer
	 */
    public Scheduler() {
		timer = new Timer();
    }

	/**
	 * genera un task a partire dal nome della classe e dai parametri
	 * e lo schedula dopo un secondo (serve a lanciare immediatamente
	 * un job in un thread separato)
	 * @param className
	 * @param params
	 * @throws ParseException
	 */
	public void scheduleCommand(String className, String[] params) throws ParseException {

		TimerTask task = createTask(className,params);
		if(task == null)
			return;
		timer.schedule(task,(long) 1000); //start in 1 second
	}

	/**
	 * genera un task a partire dal nome della classe e dai parametri
	 * e lo schedula per il momento indicato nella paramento "at" (java.util.Date)
	 * @param className
	 * @param params
	 * @param at
	 * @throws ParseException
	 */
	public void scheduleCommand(String className, String[] params, Date at) throws ParseException {

		TimerTask task = createTask(className,params);
		if(task == null)
			return;
		timer.schedule(task,at);
	}

	/**
	 * genera un task a partire dal nome della classe e dai parametri
	 * questi sono acquisiti scomponendo la stringa "line" in "parole"
	 * separate da blanks, la prima parola &egrave; la classe di esecuzione
	 * (che deve implementare l'interfaccia <code>Command</code>), le altre
	 * sono i parametri necessari alla classe per agire.
	 * Il task &egrave; schedulato per il momento indicato nella paramento "at" (java.util.Date)
	 * @param line
	 * @param at
	 * @throws ParseException
	 */
    public void scheduleCommand(String line, Date at) throws ParseException {
		StringTokenizer st = new StringTokenizer(line," ");

        // classname
        String name = st.nextToken();

        // arguments
        List list = new ArrayList();
        while(true) {
			if(st.hasMoreTokens() == false)
                break;

            list.add(st.nextToken());
        }

        TimerTask task = createTask(name,(String[]) list.toArray(new String[0]));
        if(task == null)
            return;
		timer.schedule(task,at);
		
    }

	/**
	 * crea il TimerTask a partire dal nome della classe di esecuzione
	 * e dell'arrai dei parametri
	 * @param name
	 * @param args
	 * @return TimerTask
	 */
	private TimerTask createTask(String name,String args[]) {
        try {
	        Class dclass = Class.forName(name);

            Class classes[] = dclass.getInterfaces();
            final String cname = Command.class.getName();

            boolean ok = false;
            for(int k=0;k<classes.length;k++) {
				if(classes[k].getName().compareTo(cname) == 0)
                    ok = true;
            }

            if(ok == false) {
                System.out.println("Interface Command not implemented");
                return null;
            }

        	Command cmd = (Command) dclass.newInstance();
            return new ScheduledThreadedTask(cmd,args);
			//return new ScheduledTask(cmd,args);
		} catch(Exception e) {
			System.out.println("Task creation : " + e.getMessage());
            return null;
        }
    }


}
